function FRF = Proposed_Tank_MIX_function(L,r0,r,h,number_of_modes,omega)
%% model of a water tank
%
% Calculates the Green Function (Frequency Response Function) inside
% a  water tank with given dimensions L = [Lx Ly Lz] between the source
% position r0 = [x0 y0 z0] and receiver position r = [x y z].
%
% FRF = small_tank_FRF_function(L,r0,r,h,number_of_modes,omega)
%
% inputs:    L               ... water tank dimensions [Lx, Ly, Lz]
%            r0              ... source position [x0 y0 z0]
%            r               ... receiver position [x y z]
%            h               ... wall thickness
%            number_of_modes ... number of modes
%            omega           ... angular frequency (2*pi*f)
%
% output:    FRF             ... Frequency Repsonse Function
%%
% wall properties
c_w = 2690; rho_w = 1180; Zw = c_w * rho_w;
% liquid properties
%c_l = 1488; rho_l = 998; Zl = c_l * rho_l;
c_l = 1500; rho_l = 1000; Zl = c_l * rho_l;
% air properties
c_a = 340;  rho_a = 1.255; Za = c_a * rho_a;
% wave number
kl = omega./c_l; % liquid
%%
psum = 0;
for mx=1:number_of_modes
    display(['mode n.' num2str(mx)]);
    for my=1:number_of_modes
        for mz=1:number_of_modes
            % Dirichlet (0th order) modal wave-number components (in
            % liquid)[N-2]
            kmx = mx*pi/L(1);
            kmy = my*pi/L(2);
            kmz = mz*pi/L(3);
            % (0th order) modulus of the  wave-number (in liquid)[N-2]
            km = sqrt(kmx.^2 + kmy.^2 + kmz.^2);
            % (0th order) modulus of modal wave-numbers in air (kam) and wall (kwm)
            kwm = c_l/c_w*km.*(1-1i*0.022); % 0.022 losses in walls
            kam = c_l/c_a*km;

            % modal wave-number components in wall
            % (for wall)
            kwmx = -1i*sqrt(-kwm.^2 + ((kmy).^2 + (kmz).^2));
            kwmy = -1i*sqrt(-kwm.^2 + ((kmx).^2 + (kmz).^2));
            kwmz = -1i*sqrt(-kwm.^2 + ((kmx).^2 + (kmy).^2));

            % modal wave-number components in air
            % for air
            kamx = -1i*sqrt(-kam.^2 + ((kmy).^2 + (kmz).^2));
            kamy = -1i*sqrt(-kam.^2 + ((kmx).^2 + (kmz).^2));
            kamz = -1i*sqrt(-kam.^2 + ((kmx).^2 + (kmy).^2));

            Ealx = Za./Zl .* (kmx./km)./(kamx./kam);
            Ealy = Za./Zl .* (kmy./km)./(kamy./kam);
            Ealz = Za./Zl .* (kmz./km)./(kamz./kam);

            Ewlx = Zw./Zl .* (kmx./km)./(kwmx./kwm);
            Ewly = Zw./Zl .* (kmy./km)./(kwmy./kwm);
            Ewlz = Zw./Zl .* (kmz./km)./(kwmz./kwm);

            Eawx = Za./Zw .* (kwmx./kwm)./(kamx./kam);
            Eawy = Za./Zw .* (kwmy./kwm)./(kamy./kam);
            Eawz = Za./Zw .* (kwmz./kwm)./(kamz./kam);

            % modal specific impedance components
            zetax = (Ealx + 1i*Ewlx.*tan(kwmx*h))./(1 + Eawx.*tan(kwmx*h));
            zetay = (Ealy + 1i*Ewly.*tan(kwmy*h))./(1 + Eawy.*tan(kwmy*h));

            kmx = (mx*pi +  1i .* (zetax + zetax))./L(1);
            kmy = (my*pi +  1i .* (zetay + zetay))./L(2);
            km2 = kmx.^2 + kmy.^2 + kmz.^2;

            %Change Z-domain
            Psi0 = sin(kmx*r0(1) - 1i.*zetax) .* sin(kmy*r0(2) - 1i.*zetay) .* sin(kmz*r0(3));
            Psir  = sin(kmx*r(1)  - 1i.*zetax) .* sin(kmy*r(2)  - 1i.*zetay) .* sin(kmz*r(3));

            psum = psum + Psir.*Psi0 ./ (km2-kl.^2);

        end
    end
end

FRF = sqrt(2^3./prod(L)) * psum;